@@ -6,15 +6,7 @@
 Represents an enumeration using a widget.
 """
 
-try:
-    from collections.abc import Iterable, Mapping
-except ImportError:
-    from collections import Iterable, Mapping # py2
-
-try:
-    from itertools import izip
-except ImportError:  #python3.x
-    izip = zip
+from collections.abc import Iterable, Mapping
 from itertools import chain
 
 from .widget_description import DescriptionWidget, DescriptionStyle
@@ -23,18 +15,19 @@ from .widget_core import CoreWidget
 from .widget_style import Style
 from .trait_types import InstanceDict, TypedTuple
 from .widget import register, widget_serialization
+from .widget_int import SliderStyle
 from .docutils import doc_subst
 from traitlets import (Unicode, Bool, Int, Any, Dict, TraitError, CaselessStrEnum,
                        Tuple, Union, observe, validate)
-from ipython_genutils.py3compat import unicode_type
 
 _doc_snippets = {}
 _doc_snippets['selection_params'] = """
     options: list
         The options for the dropdown. This can either be a list of values, e.g.
-        ``['Galileo', 'Brahe', 'Hubble']`` or ``[0, 1, 2]``, or a list of
+        ``['Galileo', 'Brahe', 'Hubble']`` or ``[0, 1, 2]``, a list of
         (label, value) pairs, e.g.
-        ``[('Galileo', 0), ('Brahe', 1), ('Hubble', 2)]``.
+        ``[('Galileo', 0), ('Brahe', 1), ('Hubble', 2)]``, or a Mapping between
+        labels and values, e.g., ``{'Galileo': 0, 'Brahe': 1, 'Hubble': 2}``.
 
     index: int
         The index of the current selection.
@@ -61,14 +54,12 @@ _doc_snippets['selection_params'] = """
 _doc_snippets['multiple_selection_params'] = """
     options: dict or list
         The options for the dropdown. This can either be a list of values, e.g.
-        ``['Galileo', 'Brahe', 'Hubble']`` or ``[0, 1, 2]``, a list of
+        ``['Galileo', 'Brahe', 'Hubble']`` or ``[0, 1, 2]``, or a list of
         (label, value) pairs, e.g.
-        ``[('Galileo', 0), ('Brahe', 1), ('Hubble', 2)]``,
-        or a dictionary mapping the labels to the values, e.g. ``{'Galileo': 0,
-        'Brahe': 1, 'Hubble': 2}``. The labels are the strings that will be
-        displayed in the UI, representing the actual Python choices, and should
-        be unique. If this is a dictionary, the order in which they are
-        displayed is not guaranteed.
+        ``[('Galileo', 0), ('Brahe', 1), ('Hubble', 2)]``, or a Mapping between
+        labels and values, e.g., ``{'Galileo': 0, 'Brahe': 1, 'Hubble': 2}``.
+        The labels are the strings that will be displayed in the UI,
+        representing the actual Python choices, and should be unique.
 
     index: iterable of int
         The indices of the options that are selected.
@@ -106,6 +97,13 @@ _doc_snippets['slider_params'] = """
 """
 
 
+def _exhaust_iterable(x):
+    """Exhaust any non-mapping iterable into a tuple"""
+    if isinstance(x, Iterable) and not isinstance(x, Mapping):
+        return tuple(x)
+    return x
+
+
 def _make_options(x):
     """Standardize the options tuple format.
 
@@ -114,22 +112,20 @@ def _make_options(x):
     The input can be
     * an iterable of (label, value) pairs
     * an iterable of values, and labels will be generated
+    * a Mapping between labels and values
     """
-    # Check if x is a mapping of labels to values
     if isinstance(x, Mapping):
-        import warnings
-        warnings.warn("Support for mapping types has been deprecated and will be dropped in a future release.", DeprecationWarning)
-        return tuple((unicode_type(k), v) for k, v in x.items())
+        x = x.items()
 
     # only iterate once through the options.
     xlist = tuple(x)
 
     # Check if x is an iterable of (label, value) pairs
     if all((isinstance(i, (list, tuple)) and len(i) == 2) for i in xlist):
-        return tuple((unicode_type(k), v) for k, v in xlist)
+        return tuple((str(k), v) for k, v in xlist)
 
     # Otherwise, assume x is an iterable of values
-    return tuple((unicode_type(i), i) for i in xlist)
+    return tuple((str(i), i) for i in xlist)
 
 def findvalue(array, value, compare = lambda x, y: x == y):
     "A function that uses the compare function to return a value from the list."
@@ -141,10 +137,10 @@ def findvalue(array, value, compare = lambda x, y: x == y):
 class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
     """Base class for Selection widgets
 
-    ``options`` can be specified as a list of values, list of (label, value)
-    tuples, or a dict of {label: value}. The labels are the strings that will be
-    displayed in the UI, representing the actual Python choices, and should be
-    unique. If labels are not specified, they are generated from the values.
+    ``options`` can be specified as a list of values or a list of (label, value)
+    tuples. The labels are the strings that will be displayed in the UI,
+    representing the actual Python choices, and should be unique.
+    If labels are not specified, they are generated from the values.
 
     When programmatically setting the value, a reverse lookup is performed
     among the options to check that the value is valid. The reverse lookup uses
@@ -158,7 +154,7 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
     index = Int(None, help="Selected index", allow_none=True).tag(sync=True)
 
     options = Any((),
-    help="""Iterable of values, (label, value) pairs, or a mapping of {label: value} pairs that the user can select.
+    help="""Iterable of values, (label, value) pairs, or Mapping between labels and values that the user can select.
 
     The labels are the strings that will be displayed in the UI, representing the
     actual Python choices, and should be unique.
@@ -176,25 +172,24 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
         # We have to make the basic options bookkeeping consistent
         # so we don't have errors the first time validators run
         self._initializing_traits_ = True
-        options = _make_options(kwargs.get('options', ()))
-        self._options_full = options
-        self.set_trait('_options_labels', tuple(i[0] for i in options))
-        self._options_values = tuple(i[1] for i in options)
+        kwargs['options'] = _exhaust_iterable(kwargs.get('options', ()))
+        self._options_full = _make_options(kwargs['options'])
+        self._propagate_options(None)
 
         # Select the first item by default, if we can
         if 'index' not in kwargs and 'value' not in kwargs and 'label' not in kwargs:
+            options = self._options_full
             nonempty = (len(options) > 0)
             kwargs['index'] = 0 if nonempty else None
             kwargs['label'], kwargs['value'] = options[0] if nonempty else (None, None)
 
-        super(_Selection, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._initializing_traits_ = False
 
     @validate('options')
     def _validate_options(self, proposal):
         # if an iterator is provided, exhaust it
-        if isinstance(proposal.value, Iterable) and not isinstance(proposal.value, Mapping):
-            proposal.value = tuple(proposal.value)
+        proposal.value = _exhaust_iterable(proposal.value)
         # throws an error if there is a problem converting to full form
         self._options_full = _make_options(proposal.value)
         return proposal.value
@@ -205,6 +200,12 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
         options = self._options_full
         self.set_trait('_options_labels', tuple(i[0] for i in options))
         self._options_values = tuple(i[1] for i in options)
+
+        if self.index is None:
+            # Do nothing, we don't want to force a selection if
+            # the options list changed
+            return
+
         if self._initializing_traits_ is not True:
             if len(options) > 0:
                 if self.index == 0:
@@ -246,7 +247,7 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
     def _propagate_value(self, change):
         if change.new is None:
             index = None
-        elif self.index is not None and self._options_values[self.index] == change.new:
+        elif self.index is not None and self.equals(self._options_values[self.index], change.new):
             index = self.index
         else:
             index = self._options_values.index(change.new)
@@ -271,7 +272,7 @@ class _Selection(DescriptionWidget, ValueWidget, CoreWidget):
             self.index = index
 
     def _repr_keys(self):
-        keys = super(_Selection, self)._repr_keys()
+        keys = super()._repr_keys()
         # Include options manually, as it isn't marked as synced:
         for key in sorted(chain(keys, ('options',))):
             if key == 'index' and self.index == 0:
@@ -300,7 +301,7 @@ class _MultipleSelection(DescriptionWidget, ValueWidget, CoreWidget):
     index = TypedTuple(trait=Int(), help="Selected indices").tag(sync=True)
 
     options = Any((),
-    help="""Iterable of values, (label, value) pairs, or a mapping of {label: value} pairs that the user can select.
+    help="""Iterable of values, (label, value) pairs, or Mapping between labels and values that the user can select.
 
     The labels are the strings that will be displayed in the UI, representing the
     actual Python choices, and should be unique.
@@ -318,18 +319,16 @@ class _MultipleSelection(DescriptionWidget, ValueWidget, CoreWidget):
         # We have to make the basic options bookkeeping consistent
         # so we don't have errors the first time validators run
         self._initializing_traits_ = True
-        options = _make_options(kwargs.get('options', ()))
-        self._full_options = options
-        self.set_trait('_options_labels', tuple(i[0] for i in options))
-        self._options_values = tuple(i[1] for i in options)
+        kwargs['options'] = _exhaust_iterable(kwargs.get('options', ()))
+        self._options_full = _make_options(kwargs['options'])
+        self._propagate_options(None)
 
-        super(_MultipleSelection, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._initializing_traits_ = False
 
     @validate('options')
     def _validate_options(self, proposal):
-        if isinstance(proposal.value, Iterable) and not isinstance(proposal.value, Mapping):
-            proposal.value = tuple(proposal.value)
+        proposal.value = _exhaust_iterable(proposal.value)
         # throws an error if there is a problem converting to full form
         self._options_full = _make_options(proposal.value)
         return proposal.value
@@ -389,10 +388,9 @@ class _MultipleSelection(DescriptionWidget, ValueWidget, CoreWidget):
             self.index = index
 
     def _repr_keys(self):
-        keys = super(_MultipleSelection, self)._repr_keys()
+        keys = super()._repr_keys()
         # Include options manually, as it isn't marked as synced:
-        for key in sorted(chain(keys, ('options',))):
-            yield key
+        yield from sorted(chain(keys, ('options',)))
 
 
 @register
@@ -528,12 +526,11 @@ class _SelectionNonempty(_Selection):
     def __init__(self, *args, **kwargs):
         if len(kwargs.get('options', ())) == 0:
             raise TraitError('options must be nonempty')
-        super(_SelectionNonempty, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @validate('options')
     def _validate_options(self, proposal):
-        if isinstance(proposal.value, Iterable) and not isinstance(proposal.value, Mapping):
-            proposal.value = tuple(proposal.value)
+        proposal.value = _exhaust_iterable(proposal.value)
         self._options_full = _make_options(proposal.value)
         if len(self._options_full) == 0:
             raise TraitError("Option list must be nonempty")
@@ -552,12 +549,11 @@ class _MultipleSelectionNonempty(_MultipleSelection):
     def __init__(self, *args, **kwargs):
         if len(kwargs.get('options', ())) == 0:
             raise TraitError('options must be nonempty')
-        super(_MultipleSelectionNonempty, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @validate('options')
     def _validate_options(self, proposal):
-        if isinstance(proposal.value, Iterable) and not isinstance(proposal.value, Mapping):
-            proposal.value = tuple(proposal.value)
+        proposal.value = _exhaust_iterable(proposal.value)
         # throws an error if there is a problem converting to full form
         self._options_full = _make_options(proposal.value)
         if len(self._options_full) == 0:
@@ -586,13 +582,18 @@ class SelectionSlider(_SelectionNonempty):
         help="Display the current selected label next to the slider").tag(sync=True)
     continuous_update = Bool(True,
         help="Update the value of the widget as the user is holding the slider.").tag(sync=True)
+    behavior = CaselessStrEnum(values=['drag-tap', 'drag-snap', 'tap', 'drag', 'snap'],
+        default_value='drag-tap', help="Slider dragging behavior.").tag(sync=True)
+
+    style = InstanceDict(SliderStyle).tag(sync=True, **widget_serialization)
+
 
 @register
 @doc_subst(_doc_snippets)
 class SelectionRangeSlider(_MultipleSelectionNonempty):
     """
     Slider to select multiple contiguous items from a list.
-    
+
     The index, value, and label attributes contain the start and end of
     the selection range, not all items in the range.
 
@@ -635,3 +636,7 @@ class SelectionRangeSlider(_MultipleSelectionNonempty):
         help="Display the current selected label next to the slider").tag(sync=True)
     continuous_update = Bool(True,
         help="Update the value of the widget as the user is holding the slider.").tag(sync=True)
+
+    style = InstanceDict(SliderStyle).tag(sync=True, **widget_serialization)
+    behavior = CaselessStrEnum(values=['drag-tap', 'drag-snap', 'tap', 'drag', 'snap'],
+        default_value='drag-tap', help="Slider dragging behavior.").tag(sync=True)
